<!--
Project: Content Management System.
Component: Writer (File 5 of 5).
Filename: "finalize_assets.php".
Purpose: Loops through metadata file and writes out all assets and their associated metadata to a valid XML file.
-->

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
 <head>
  <title> Simple Content Management System </title>
 </head>

 <body>

<h1>Simple Content Management System</h1>
<h2>Step 3: Generate XML Document</h2>

<?php

/* 
The $bufferFilename variable is used to store the name of the temporary file we will be reading from. For this exercise, this file needs to be in the same directory as this script, but it could be changed by including the directory path as part of this variable.  The $xmlOutputFilename will be the name of the file used to store our XML collection.  Note that this default filename will automatically append the current month (in the format Jan, Feb, Mar, etc.) day, and year (in four digit format) to the latter half of the filename.
*/

$bufferFilename = "metadata_buffer.txt";
$metaDataDirectory = "metadata/";
$xmlOutputFilename = "asset_collection_".date("MdY").".xml";

/* This function writes the beginning of the XML file using UTF-8 (Latin) encoding */
function startXMLdocument($xmlString) 
	{
	$xmlString = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
	$xmlString .= "<!-- XML Asset Collection Document -->\n";
	$xmlString .= "<!-- Automatically generated by Content Management System -->\n";
	$xmlString .= "<!-- Created on ".date("MdY")." -->\n";
	$xmlString .= "<?xml-stylesheet type=\"text/xsl\" href=\"asset_collection_transform.xsl\"?>\n\n";
	return($xmlString);
	}

/* 
This function writes the document type definition (DTD) as well as the top level asset_collection container to the XML file 
*/
function insertDTD($xmlString) 
	{
	$xmlString .= "<!DOCTYPE asset_collection [\n";
	$xmlString .= "\t<!ELEMENT asset_collection (asset)*>\n";
	$xmlString .= "\t<!ELEMENT asset (id,filename,name,category,years,media,description)>\n";
	$xmlString .= "\t<!ELEMENT id (#PCDATA)>\n";
	$xmlString .= "\t<!ELEMENT filename (#PCDATA)>\n";
	$xmlString .= "\t<!ELEMENT name (#PCDATA)>\n";
	$xmlString .= "\t<!ELEMENT category (#PCDATA)>\n";
	$xmlString .= "\t<!ELEMENT years (#PCDATA)>\n";
	$xmlString .= "\t<!ELEMENT media (#PCDATA)>\n";
	$xmlString .= "\t<!ELEMENT description (#PCDATA)>\n";
	$xmlString .= "]>\n\n";
	$xmlString .= "<asset_collection>\n";
	return($xmlString);
	}

/* 
This function closes the asset_collection container 
*/
function endXMLdocument($xmlString)
	{
	$xmlString .= "</asset_collection>\n";
	return($xmlString);
	}

/* 
This function will loop through the temporary buffer file and find each asset and its metadata.  It can tell where one asset ends and another begins because they are each separated by newlines (or the \n character sequence).  It can also tell where one element ends and the next one begins by the *** character sequence.  Each asset is converted into an XML element with its values written out into the XML file as character data.
*/
function addAssetCollection($filename,$xmlString)
	{
	if ($filecontents = file($filename))
		{
		$xmlString = startXMLdocument($xmlString);
		$xmlString = insertDTD($xmlString);
		foreach($filecontents as $singleLine)
			{
			$line = explode("\n",$singleLine);
			$units = explode("***",$line[0]);
			$xmlString .= "\t\t<asset>\n";
			$xmlString .= "\t\t\t<id>".$units[0]."</id>\n";
			$xmlString .= "\t\t\t<filename>".$units[1]."</filename>\n";	
			$xmlString .= "\t\t\t<name>".$units[2]."</name>\n";
			$xmlString .= "\t\t\t<category>".$units[3]."</category>\n";
			$xmlString .= "\t\t\t<years>".$units[4]."</years>\n";
			$xmlString .= "\t\t\t<media>".$units[5]."</media>\n";
			$xmlString .= "\t\t\t<description>".$units[6]."</description>\n";		
			$xmlString .= "\t\t</asset>\n";			
			}
		$xmlString = endXMLdocument($xmlString);
		return($xmlString);
		}
	else
		{
		echo "Unable to open file!";
		}	
	}

/*
This function handles the actual writing of the XML document to a permanent file.  It will use the same metadata directory and filename as specified by the variables at the top of this script. 
*/
function writeXMLFile($metaDataDirectory, $fileName, $xmlString)
	{
	$fp = fopen($metaDataDirectory.$fileName, "w");
	if ($bytesWrittenToFile = fwrite($fp, $xmlString))
		{
		echo "XML file created successfully!<br />";
		return TRUE;
		}
	else
		{
		echo "There was a problem writing to file.<br />";
		echo "Here is the XML content: <br /><pre>$xmlString</pre>";
		return FALSE;
		}
	}

/*
Here is where the addAssetCollection is "called" to create the XML string that will be written to the permanent file.  For now, the entire XML document will be stored in a temporary string variable named $xml.
*/
$xml = addAssetCollection($bufferFilename, $xmlString);

/*
This segment of code writes the XML string to a permanent file.  If it is unable to create the file, the error message defined in the writeXMLFile function will be displayed.
*/
if (writeXMLFile($metaDataDirectory, $xmlOutputFilename, $xml))
	{
	echo "<br />";
	echo "Click here to <a href=\"".$metaDataDirectory.$xmlOutputFilename."\">view your collection using an XSL transform</a>.<br />";
	echo "Click here to <a href=\"upload_asset.html\">begin a new asset collection</a>.<br />";
	echo "Click here to <a href=\"select_xml.php\">select an XML file to parse using the CMS parser</a>.";
	}

?>

</body>
</html>

<!-- End of "finalize_assets.php" file -->